@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009, Nokia (urho.konttori@nokia.com)
+ * Copyright (C) 2009, Nokia <ivan.frade@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -37,7 +37,7 @@
 /**
  * SECTION:tracker-miner-fs
  * @short_description: Abstract base class for filesystem miners
- * @include: libtracker-miner/tracker-miner-fs.h
+ * @include: libtracker-miner/tracker-miner.h
  *
  * #TrackerMinerFS is an abstract base class for miners that collect data
  * from the filesystem, all the filesystem crawling and monitoring is
@@ -54,7 +54,8 @@ typedef struct {
 
 typedef struct {
 	GFile    *file;
-	gboolean  recurse;
+	guint     recurse : 1;
+	guint     ref_count : 7;
 } DirectoryData;
 
 typedef struct {
@@ -73,6 +74,11 @@ typedef struct {
 	gboolean get_mime;
 } SparqlQueryData;
 
+typedef struct {
+	GMainLoop *main_loop;
+	GHashTable *values;
+} CacheQueryData;
+
 typedef struct {
 	GMainLoop *main_loop;
 	GString   *sparql;
@@ -102,6 +108,8 @@ struct TrackerMinerFSPrivate {
 
 	GQuark          quark_ignore_file;
 
+	GList          *config_directories;
+
 	GList          *directories;
 	DirectoryData  *current_directory;
 
@@ -119,6 +127,12 @@ struct TrackerMinerFSPrivate {
 	GFile          *current_parent;
 	gchar          *current_parent_urn;
 
+	/* Folder contents' mtime cache */
+	GHashTable     *mtime_cache;
+
+	/* File -> iri cache */
+	GHashTable     *iri_cache;
+
 	/* Status */
 	guint           been_started : 1;
 	guint           been_crawled : 1;
@@ -191,7 +205,8 @@ static void           miner_ignore_next_update            (TrackerMiner
                                                            const GStrv           subjects);
 static DirectoryData *directory_data_new                  (GFile                *file,
                                                            gboolean              recurse);
-static void           directory_data_free                 (DirectoryData        *dd);
+static DirectoryData *directory_data_ref                  (DirectoryData        *dd);
+static void           directory_data_unref                (DirectoryData        *dd);
 static ItemMovedData *item_moved_data_new                 (GFile                *file,
                                                            GFile                *source_file);
 static void           item_moved_data_free                (ItemMovedData        *data);
@@ -243,6 +258,11 @@ static void           item_update_children_uri            (TrackerMinerFS
                                                            const gchar          *uri);
 static void           crawled_directory_data_free         (CrawledDirectoryData *data);
 
+static gboolean       should_recurse_for_directory            (TrackerMinerFS *fs,
+							       GFile          *file);
+static void           tracker_miner_fs_directory_add_internal (TrackerMinerFS *fs,
+							       GFile          *file);
+
 
 static guint signals[LAST_SIGNAL] = { 0, };
 
@@ -510,6 +530,11 @@ tracker_miner_fs_init (TrackerMinerFS *object)
 	                  object);
 
 	priv->quark_ignore_file = g_quark_from_static_string ("tracker-ignore-file");
+
+	priv->iri_cache = g_hash_table_new_full (g_file_hash,
+	                                         (GEqualFunc) g_file_equal,
+	                                         (GDestroyNotify) g_object_unref,
+	                                         (GDestroyNotify) g_free);
 }
 
 static ProcessData *
@@ -605,10 +630,15 @@ fs_finalize (GObject *object)
 	g_free (priv->current_parent_urn);
 
 	if (priv->directories) {
-		g_list_foreach (priv->directories, (GFunc) directory_data_free, NULL);
+		g_list_foreach (priv->directories, (GFunc) directory_data_unref, NULL);
 		g_list_free (priv->directories);
 	}
 
+	if (priv->config_directories) {
+		g_list_foreach (priv->config_directories, (GFunc) directory_data_unref, NULL);
+		g_list_free (priv->config_directories);
+	}
+
 	g_queue_foreach (priv->crawled_directories, (GFunc) crawled_directory_data_free, NULL);
 	g_queue_free (priv->crawled_directories);
 
@@ -629,6 +659,14 @@ fs_finalize (GObject *object)
 
 	g_hash_table_unref (priv->items_ignore_next_update);
 
+	if (priv->mtime_cache) {
+		g_hash_table_unref (priv->mtime_cache);
+	}
+
+	if (priv->iri_cache) {
+		g_hash_table_unref (priv->iri_cache);
+	}
+
 	G_OBJECT_CLASS (tracker_miner_fs_parent_class)->finalize (object);
 }
 
@@ -704,7 +742,7 @@ miner_started (TrackerMiner *miner)
 
 	g_object_set (miner,
 	              "progress", 0.0,
-	              "status", _("Initializing"),
+	              "status", "Initializing",
 	              NULL);
 
 	crawl_directories_start (fs);
@@ -715,7 +753,7 @@ miner_stopped (TrackerMiner *miner)
 {
 	g_object_set (miner,
 	              "progress", 1.0,
-	              "status", _("Idle"),
+	              "status", "Idle",
 	              NULL);
 }
 
@@ -787,19 +825,31 @@ directory_data_new (GFile    *file,
 
 	dd->file = g_object_ref (file);
 	dd->recurse = recurse;
+	dd->ref_count = 1;
 
 	return dd;
 }
 
+static DirectoryData *
+directory_data_ref (DirectoryData *dd)
+{
+	dd->ref_count++;
+	return dd;
+}
+
 static void
-directory_data_free (DirectoryData *dd)
+directory_data_unref (DirectoryData *dd)
 {
 	if (!dd) {
 		return;
 	}
 
-	g_object_unref (dd->file);
-	g_slice_free (DirectoryData, dd);
+	dd->ref_count--;
+
+	if (dd->ref_count == 0) {
+		g_object_unref (dd->file);
+		g_slice_free (DirectoryData, dd);
+	}
 }
 
 static void
@@ -857,7 +907,7 @@ process_stop (TrackerMinerFS *fs)
 
 	g_object_set (fs,
 	              "progress", 1.0,
-	              "status", _("Idle"),
+	              "status", "Idle",
 	              NULL);
 
 	g_signal_emit (fs, signals[FINISHED], 0,
@@ -928,6 +978,22 @@ sparql_update_cb (GObject      *object,
 			 */
 			tracker_miner_commit (TRACKER_MINER (fs), NULL, commit_cb, NULL);
 		}
+
+		if (fs->private->current_parent) {
+			GFile *parent;
+
+			parent = g_file_get_parent (data->file);
+
+			if (g_file_equal (parent, fs->private->current_parent) &&
+			    g_hash_table_lookup (fs->private->iri_cache, data->file) == NULL) {
+				/* Item is processed, add an empty element for the processed GFile,
+				 * in case it is again processed before the cache expires
+				 */
+				g_hash_table_insert (fs->private->iri_cache, g_object_ref (data->file), NULL);
+			}
+
+			g_object_unref (parent);
+		}
 	}
 
 	priv->processing_pool = g_list_remove (priv->processing_pool, data);
@@ -1025,6 +1091,115 @@ item_query_exists (TrackerMinerFS  *miner,
 	return result;
 }
 
+static void
+cache_query_cb (GObject	     *object,
+		GAsyncResult *result,
+		gpointer      user_data)
+{
+	const GPtrArray *query_results;
+	TrackerMiner *miner;
+	CacheQueryData *data;
+	GError *error = NULL;
+	guint i;
+
+	data = user_data;
+	miner = TRACKER_MINER (object);
+	query_results = tracker_miner_execute_sparql_finish (miner, result, &error);
+
+	g_main_loop_quit (data->main_loop);
+
+	if (G_UNLIKELY (error)) {
+		g_critical ("Could not execute cache query: %s", error->message);
+		g_error_free (error);
+		return;
+	}
+
+	for (i = 0; i < query_results->len; i++) {
+		GFile *file;
+		GStrv strv;
+
+		strv = g_ptr_array_index (query_results, i);
+		file = g_file_new_for_uri (strv[0]);
+
+		g_hash_table_insert (data->values, file, g_strdup (strv[1]));
+	}
+}
+
+static void
+ensure_iri_cache (TrackerMinerFS *fs,
+                  GFile          *parent)
+{
+	gchar *query, *uri;
+	CacheQueryData data;
+
+	g_hash_table_remove_all (fs->private->iri_cache);
+
+	uri = g_file_get_uri (parent);
+
+	g_debug ("Generating IRI cache for folder: %s", uri);
+
+	query = g_strdup_printf ("SELECT ?uri ?u { "
+	                         "  ?u nie:url ?uri . "
+	                         "  FILTER (tracker:uri-is-parent (\"%s\", ?uri)) "
+	                         "}",
+	                         uri);
+	g_free (uri);
+
+	data.main_loop = g_main_loop_new (NULL, FALSE);
+	data.values = g_hash_table_ref (fs->private->iri_cache);
+
+	tracker_miner_execute_sparql (TRACKER_MINER (fs),
+	                              query,
+	                              NULL,
+	                              cache_query_cb,
+	                              &data);
+
+	g_main_loop_run (data.main_loop);
+
+	g_main_loop_unref (data.main_loop);
+	g_hash_table_unref (data.values);
+	g_free (query);
+}
+
+static const gchar *
+iri_cache_lookup (TrackerMinerFS *fs,
+                  GFile          *file)
+{
+	gpointer value;
+	const gchar *iri;
+
+	if (!g_hash_table_lookup_extended (fs->private->iri_cache, file, NULL, &value)) {
+		/* Item doesn't exist in cache */
+		return NULL;
+	}
+
+	iri = value;
+
+	if (!iri) {
+		gchar *query_iri;
+
+		/* Cache miss, this item was added after the last
+		 * iri cache update, so query it independently
+		 */
+		if (item_query_exists (fs, file, &query_iri, NULL)) {
+			g_hash_table_insert (fs->private->iri_cache,
+			                     g_object_ref (file), query_iri);
+		} else {
+			g_hash_table_remove (fs->private->iri_cache, file);
+			iri = NULL;
+		}
+	}
+
+	return iri;
+}
+
+static void
+iri_cache_invalidate (TrackerMinerFS *fs,
+                      GFile          *file)
+{
+	g_hash_table_remove (fs->private->iri_cache, file);
+}
+
 static gboolean
 do_process_file (TrackerMinerFS *fs,
 		 ProcessData    *data)
@@ -1100,11 +1275,7 @@ item_add_or_update_cb (TrackerMinerFS *fs,
 
 			do_process_file (fs, data);
 		} else {
-			if (error->code == G_IO_ERROR_NOT_FOUND) {
-				g_message ("Could not process '%s': %s", uri, error->message);
-			} else {
-				g_critical ("Could not process '%s': %s", uri, error->message);
-			}
+                        g_message ("Could not process '%s': %s", uri, error->message);
 
 			fs->private->total_files_notified_error++;
 			fs->private->processing_pool =
@@ -1142,7 +1313,7 @@ item_add_or_update (TrackerMinerFS *fs,
 	gboolean retval;
 	ProcessData *data;
 	GFile *parent;
-	gchar *urn;
+	const gchar *urn;
 	const gchar *parent_urn = NULL;
 
 	priv = fs->private;
@@ -1152,8 +1323,6 @@ item_add_or_update (TrackerMinerFS *fs,
 	sparql = tracker_sparql_builder_new_update ();
 	g_object_ref (file);
 
-	item_query_exists (fs, file, &urn, NULL);
-
 	parent = g_file_get_parent (file);
 
 	if (parent) {
@@ -1170,18 +1339,21 @@ item_add_or_update (TrackerMinerFS *fs,
 
 			g_free (fs->private->current_parent_urn);
 
-			if (item_query_exists (fs, parent, &fs->private->current_parent_urn, NULL))
-				fs->private->current_parent = g_object_ref (parent);
-			else {
-				fs->private->current_parent = NULL;
+			fs->private->current_parent = g_object_ref (parent);
+
+			if (!item_query_exists (fs, parent, &fs->private->current_parent_urn, NULL)) {
 				fs->private->current_parent_urn = NULL;
 			}
+
+			ensure_iri_cache (fs, parent);
 		}
 
 		parent_urn = fs->private->current_parent_urn;
 		g_object_unref (parent);
 	}
 
+	urn = iri_cache_lookup (fs, file);
+
 	data = process_data_new (file, urn, parent_urn, cancellable, sparql);
 	priv->processing_pool = g_list_prepend (priv->processing_pool, data);
 
@@ -1209,10 +1381,11 @@ item_remove (TrackerMinerFS *fs,
              GFile          *file)
 {
 	GString *sparql;
-	gchar *uri, *slash_uri;
+	gchar *uri;
 	gchar *mime = NULL;
 	ProcessData *data;
 
+	iri_cache_invalidate (fs, file);
 	uri = g_file_get_uri (file);
 
 	g_debug ("Removing item: '%s' (Deleted from filesystem)",
@@ -1229,12 +1402,6 @@ item_remove (TrackerMinerFS *fs,
 
 	g_free (mime);
 
-	if (!g_str_has_suffix (uri, "/")) {
-		slash_uri = g_strconcat (uri, "/", NULL);
-	} else {
-		slash_uri = g_strdup (uri);
-	}
-
 	sparql = g_string_new ("");
 
 	/* Delete all children */
@@ -1243,16 +1410,16 @@ item_remove (TrackerMinerFS *fs,
 	                        "  ?child a rdfs:Resource "
 	                        "} WHERE {"
 	                        "  ?child nie:url ?u . "
-	                        "  FILTER (fn:starts-with (?u, '%s')) "
+	                        "  FILTER (tracker:uri-is-descendant (\"%s\", ?u)) "
 	                        "}",
-	                        slash_uri);
+	                        uri);
 
 	/* Delete resource itself */
 	g_string_append_printf (sparql,
 	                        "DELETE { "
 	                        "  ?u a rdfs:Resource "
 	                        "} WHERE { "
-	                        "  ?u nie:url '%s' "
+	                        "  ?u nie:url \"%s\" "
 	                        "}",
 	                        uri);
 
@@ -1266,7 +1433,6 @@ item_remove (TrackerMinerFS *fs,
 	                                    data);
 
 	g_string_free (sparql, TRUE);
-	g_free (slash_uri);
 	g_free (uri);
 
 	return FALSE;
@@ -1339,7 +1505,7 @@ item_ignore_next_update (TrackerMinerFS *fs,
 		                         "     nfo:fileLastModified ?unknown2 ; "
 		                         "     nfo:fileLastAccessed ?unknown3 ; "
 		                         "     nie:mimeType ?unknown4 ; "
-		                         "     nie:url '%s' "
+		                         "     nie:url \"%s\" "
 		                         "} %s",
 		                         uri, tracker_sparql_builder_get_result (sparql));
 
@@ -1406,7 +1572,7 @@ item_update_children_uri_cb (GObject      *object,
 
 			g_string_append_printf (data->sparql,
 			                        "INSERT INTO <%s> {"
-			                        "  <%s> nie:url '%s' "
+			                        "  <%s> nie:url \"%s\" "
 			                        "} ",
 			                        child_urn, child_urn, child_uri);
 
@@ -1425,18 +1591,13 @@ item_update_children_uri (TrackerMinerFS    *fs,
                           const gchar       *source_uri,
                           const gchar       *uri)
 {
-	gchar *slash_uri, *sparql;
+	gchar *sparql;
 
-	slash_uri = g_strconcat (source_uri, "/", NULL);
-
-	sparql = g_strdup_printf ("SELECT ?child ?url ?m WHERE { "
+	sparql = g_strdup_printf ("SELECT ?child ?url nie:mimeType(?child) WHERE { "
 				  "  ?child nie:url ?url . "
-				  "  OPTIONAL { "
-				  "    ?child nie:mimeType ?m "
-				  "  } . "
-				  "  FILTER (fn:starts-with (?url, \"%s\")) "
+                                  "  FILTER (tracker:uri-is-descendant (\"%s\", ?url)) "
 				  "}",
-				  slash_uri);
+				  source_uri);
 
 	tracker_miner_execute_sparql (TRACKER_MINER (fs),
 	                              sparql,
@@ -1444,7 +1605,6 @@ item_update_children_uri (TrackerMinerFS    *fs,
 	                              item_update_children_uri_cb,
 	                              move_data);
 
-	g_free (slash_uri);
 	g_free (sparql);
 }
 
@@ -1459,35 +1619,56 @@ item_move (TrackerMinerFS *fs,
 	RecursiveMoveData move_data;
 	ProcessData *data;
 	gchar *source_iri;
+	gboolean source_exists;
+
+	iri_cache_invalidate (fs, file);
+	iri_cache_invalidate (fs, source_file);
 
 	uri = g_file_get_uri (file);
 	source_uri = g_file_get_uri (source_file);
 
-	/* Get 'source' ID */
-	if (!item_query_exists (fs, source_file, &source_iri, NULL)) {
-		gboolean retval;
-
-		g_message ("Source file '%s' not found in store to move, indexing '%s' from scratch", source_uri, uri);
-
-		retval = item_add_or_update (fs, file);
-
-		g_free (source_uri);
-		g_free (uri);
-
-		return retval;
-	}
-
 	file_info = g_file_query_info (file,
 	                               G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME ","
-	                               G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
+	                               G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE ","
+				       G_FILE_ATTRIBUTE_STANDARD_TYPE,
 	                               G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
 	                               NULL, NULL);
 
+	/* Get 'source' ID */
+	source_exists = item_query_exists (fs, source_file, &source_iri, NULL);
+
 	if (!file_info) {
 		gboolean retval;
 
-		/* Destination file has gone away, ignore dest file and remove source if any */
-		retval = item_remove (fs, source_file);
+		if (source_exists) {
+			/* Destination file has gone away, ignore dest file and remove source if any */
+			retval = item_remove (fs, source_file);
+		} else {
+			/* Destination file went away, and source wasn't indexed either */
+			retval = TRUE;
+		}
+
+		g_free (source_iri);
+		g_free (source_uri);
+		g_free (uri);
+
+		return retval;
+	} else if (file_info && !source_exists) {
+		gboolean retval;
+		GFileType file_type;
+
+		g_message ("Source file '%s' not found in store to move, indexing '%s' from scratch", source_uri, uri);
+
+		file_type = g_file_info_get_file_type (file_info);
+
+		if (file_type == G_FILE_TYPE_DIRECTORY &&
+		    should_recurse_for_directory (fs, file)) {
+			/* We're dealing with a recursive directory */
+			tracker_miner_fs_directory_add_internal (fs, file);
+			retval = TRUE;
+		} else {
+			retval = item_add_or_update (fs, file);
+		}
 
 		g_free (source_uri);
 		g_free (uri);
@@ -1530,8 +1711,8 @@ item_move (TrackerMinerFS *fs,
 
 	g_string_append_printf (sparql,
 	                        "INSERT INTO <%s> {"
-	                        "  <%s> nfo:fileName '%s' ; "
-	                        "       nie:url '%s' ; "
+	                        "  <%s> nfo:fileName \"%s\" ; "
+	                        "       nie:url \"%s\" ; "
 	                        "       nie:isStoredAs <%s> "
 	                        "} ",
 	                        source_iri, source_iri,
@@ -1959,10 +2140,10 @@ item_queue_handlers_set_up (TrackerMinerFS *fs)
 
 	g_object_get (fs, "status", &status, NULL);
 
-	if (g_strcmp0 (status, _("Processing files")) != 0) {
+	if (g_strcmp0 (status, "Processing…") != 0) {
 		/* Don't spam this */
-		g_message ("Processing files...");
-		g_object_set (fs, "status", _("Processing files"), NULL);
+		g_message ("Processing…");
+		g_object_set (fs, "status", "Processing…", NULL);
 	}
 
 	g_free (status);
@@ -1973,17 +2154,80 @@ item_queue_handlers_set_up (TrackerMinerFS *fs)
 		                   fs);
 }
 
+static void
+ensure_mtime_cache (TrackerMinerFS *fs,
+                    GFile          *file)
+{
+	gchar *query, *uri;
+	CacheQueryData data;
+	GFile *parent;
+
+	if (G_UNLIKELY (!fs->private->mtime_cache)) {
+		fs->private->mtime_cache = g_hash_table_new_full (g_file_hash,
+		                                                  (GEqualFunc) g_file_equal,
+		                                                  (GDestroyNotify) g_object_unref,
+		                                                  (GDestroyNotify) g_free);
+	}
+
+	parent = g_file_get_parent (file);
+
+	if (fs->private->current_parent) {
+		if (g_file_equal (parent, fs->private->current_parent)) {
+			/* Cache is still valid */
+			g_object_unref (parent);
+			return;
+		}
+
+		g_object_unref (fs->private->current_parent);
+	}
+
+	fs->private->current_parent = parent;
+
+	g_hash_table_remove_all (fs->private->mtime_cache);
+
+	uri = g_file_get_uri (parent);
+
+	g_debug ("Generating mtime cache for folder: %s", uri);
+
+	query = g_strdup_printf ("SELECT ?uri nfo:fileLastModified (?u) { "
+	                         "  ?u nie:url ?uri . "
+	                         "  FILTER (tracker:uri-is-parent (\"%s\", ?uri)) "
+	                         "}",
+	                         uri);
+	g_free (uri);
+
+	data.main_loop = g_main_loop_new (NULL, FALSE);
+	data.values = g_hash_table_ref (fs->private->mtime_cache);
+
+	tracker_miner_execute_sparql (TRACKER_MINER (fs),
+	                              query,
+	                              NULL,
+	                              cache_query_cb,
+	                              &data);
+
+	g_main_loop_run (data.main_loop);
+
+	g_main_loop_unref (data.main_loop);
+	g_hash_table_unref (data.values);
+	g_free (query);
+}
+
 static gboolean
 should_change_index_for_file (TrackerMinerFS *fs,
                               GFile          *file)
 {
-	gboolean            uptodate;
 	GFileInfo          *file_info;
 	guint64             time;
 	time_t              mtime;
 	struct tm           t;
-	gchar              *query, *uri;
-	SparqlQueryData     data = { 0 };
+	gchar              *time_str, *lookup_time;
+
+	ensure_mtime_cache (fs, file);
+	lookup_time = g_hash_table_lookup (fs->private->mtime_cache, file);
+
+	if (!lookup_time) {
+		return TRUE;
+	}
 
 	file_info = g_file_query_info (file,
 	                               G_FILE_ATTRIBUTE_TIME_MODIFIED,
@@ -2001,42 +2245,17 @@ should_change_index_for_file (TrackerMinerFS *fs,
 	mtime = (time_t) time;
 	g_object_unref (file_info);
 
-	uri = g_file_get_uri (file);
-
 	gmtime_r (&mtime, &t);
 
-	query = g_strdup_printf ("SELECT ?file { "
-	                         "  ?file nfo:fileLastModified \"%04d-%02d-%02dT%02d:%02d:%02dZ\" . "
-	                         "  ?file nie:url \"%s\""
-	                         "}",
-	                         t.tm_year + 1900,
-	                         t.tm_mon + 1,
-	                         t.tm_mday,
-	                         t.tm_hour,
-	                         t.tm_min,
-	                         t.tm_sec,
-	                         uri);
+	time_str = g_strdup_printf ("%04d-%02d-%02dT%02d:%02d:%02dZ",
+	                            t.tm_year + 1900,
+	                            t.tm_mon + 1,
+	                            t.tm_mday,
+	                            t.tm_hour,
+	                            t.tm_min,
+	                            t.tm_sec);
 
-	data.get_mime = FALSE;
-	data.main_loop = g_main_loop_new (NULL, FALSE);
-	data.uri = uri;
-
-	tracker_miner_execute_sparql (TRACKER_MINER (fs),
-	                              query,
-	                              NULL,
-	                              sparql_query_cb,
-	                              &data);
-
-	g_main_loop_run (data.main_loop);
-	uptodate = (data.iri != NULL);
-
-	g_main_loop_unref (data.main_loop);
-	g_free (data.iri);
-	g_free (data.mime);
-	g_free (query);
-	g_free (uri);
-
-	if (uptodate) {
+	if (strcmp (time_str, lookup_time) == 0) {
 		/* File already up-to-date in the database */
 		return FALSE;
 	}
@@ -2097,8 +2316,9 @@ monitor_item_created_cb (TrackerMonitor *monitor,
 	         is_directory ? "DIR" : "FILE");
 
 	if (should_process) {
-		if (is_directory) {
-			tracker_miner_fs_directory_add (fs, file, TRUE);
+		if (is_directory &&
+		    should_recurse_for_directory (fs, file)) {
+			tracker_miner_fs_directory_add_internal (fs, file);
 		} else {
 			g_queue_push_tail (fs->private->items_created,
 			                   g_object_ref (file));
@@ -2181,7 +2401,7 @@ monitor_item_deleted_cb (TrackerMonitor *monitor,
 	 * in process_device_next()
 	 */
 	if (l && l != fs->private->current_directory) {
-		directory_data_free (l->data);
+		directory_data_unref (l->data);
 		fs->private->directories =
 			g_list_delete_link (fs->private->directories, l);
 	}
@@ -2203,7 +2423,8 @@ monitor_item_moved_cb (TrackerMonitor *monitor,
 	fs = user_data;
 
 	if (!is_source_monitored) {
-		if (is_directory) {
+		if (is_directory &&
+		    should_recurse_for_directory (fs, other_file)) {
 			gchar *path;
 
 			path = g_file_get_path (other_file);
@@ -2212,7 +2433,7 @@ monitor_item_moved_cb (TrackerMonitor *monitor,
 			         path);
 
 			/* If the source is not monitored, we need to crawl it. */
-			tracker_miner_fs_directory_add (fs, other_file, TRUE);
+			tracker_miner_fs_directory_add_internal (fs, other_file);
 
 			g_free (path);
 		}
@@ -2244,7 +2465,8 @@ monitor_item_moved_cb (TrackerMonitor *monitor,
 			/* Do nothing */
 		} else if (!source_stored) {
 			/* Source file was not stored, check dest file as new */
-			if (!is_directory) {
+			if (!is_directory ||
+			    !should_recurse_for_directory (fs, other_file)) {
 				g_queue_push_tail (fs->private->items_created,
 				                   g_object_ref (other_file));
 
@@ -2253,7 +2475,7 @@ monitor_item_moved_cb (TrackerMonitor *monitor,
 				g_debug ("Not in store:'?'->'%s' (DIR) (move monitor event, source monitored)",
 				         path);
 
-				tracker_miner_fs_directory_add (fs, other_file, TRUE);
+				tracker_miner_fs_directory_add_internal (fs, other_file);
 			}
 		} else if (!should_process_other) {
 			/* Delete old file */
@@ -2289,29 +2511,27 @@ crawler_check_directory_cb (TrackerCrawler *crawler,
                             gpointer        user_data)
 {
 	TrackerMinerFS *fs = user_data;
-	gboolean should_check, should_change_index;
-	gboolean add_monitor = TRUE;
+	gboolean should_check;
 
 	should_check = should_check_file (fs, file, TRUE);
-	should_change_index = should_change_index_for_file (fs, file);
 
-	if (!should_change_index) {
-		/* Mark the file as ignored, we still want the crawler
-		 * to iterate over its contents, but the directory hasn't
-		 * actually changed, hence this flag.
-		 */
-		g_object_set_qdata (G_OBJECT (file),
-		                    fs->private->quark_ignore_file,
-		                    GINT_TO_POINTER (TRUE));
-	}
+	if (!should_check) {
+		/* Remove monitors if any */
+		tracker_monitor_remove (fs->private->monitor, file);
+	} else {
+                gboolean should_change_index;
 
-	g_signal_emit (fs, signals[MONITOR_DIRECTORY], 0, file, &add_monitor);
+		should_change_index = should_change_index_for_file (fs, file);
 
-	/* FIXME: Should we add here or when we process the queue in
-	 * the finished sig?
-	 */
-	if (add_monitor) {
-		tracker_monitor_add (fs->private->monitor, file);
+		if (!should_change_index) {
+			/* Mark the file as ignored, we still want the crawler
+			 * to iterate over its contents, but the directory hasn't
+			 * actually changed, hence this flag.
+			 */
+			g_object_set_qdata (G_OBJECT (file),
+			                    fs->private->quark_ignore_file,
+			                    GINT_TO_POINTER (TRUE));
+		}
 	}
 
 	/* We _HAVE_ to check ALL directories because mtime updates
@@ -2332,10 +2552,24 @@ crawler_check_directory_contents_cb (TrackerCrawler *crawler,
                                      gpointer        user_data)
 {
 	TrackerMinerFS *fs = user_data;
+	gboolean add_monitor = FALSE;
 	gboolean process;
 
 	g_signal_emit (fs, signals[CHECK_DIRECTORY_CONTENTS], 0, parent, children, &process);
 
+	if (process) {
+		g_signal_emit (fs, signals[MONITOR_DIRECTORY], 0, parent, &add_monitor);
+	}
+
+	/* FIXME: Should we add here or when we process the queue in
+	 * the finished sig?
+	 */
+	if (add_monitor) {
+		tracker_monitor_add (fs->private->monitor, parent);
+	} else {
+		tracker_monitor_remove (fs->private->monitor, parent);
+	}
+
 	return process;
 }
 
@@ -2397,6 +2631,8 @@ crawled_directory_data_free (CrawledDirectoryData *data)
 			 NULL);
 	g_node_destroy (data->tree);
 
+	g_queue_free (data->nodes);
+
 	g_slice_free (CrawledDirectoryData, data);
 }
 
@@ -2458,7 +2694,7 @@ crawler_finished_cb (TrackerCrawler *crawler,
 	           was_interrupted ? "Stopped" : "Finished",
 	           g_timer_elapsed (fs->private->timer, NULL));
 
-	directory_data_free (fs->private->current_directory);
+	directory_data_unref (fs->private->current_directory);
 	fs->private->current_directory = NULL;
 
 	/* Proceed to next thing to process */
@@ -2487,6 +2723,10 @@ crawl_directories_cb (gpointer user_data)
 		return FALSE;
 	}
 
+	if (!fs->private->timer) {
+		fs->private->timer = g_timer_new ();
+	}
+
 	fs->private->current_directory = fs->private->directories->data;
 	fs->private->directories = g_list_remove (fs->private->directories,
 	                                          fs->private->current_directory);
@@ -2494,14 +2734,18 @@ crawl_directories_cb (gpointer user_data)
 	path = g_file_get_path (fs->private->current_directory->file);
 
 	if (fs->private->current_directory->recurse) {
-		str = g_strdup_printf (_("Crawling recursively directory '%s'"), path);
+		str = g_strdup_printf ("Crawling recursively directory '%s'", path);
 	} else {
-		str = g_strdup_printf (_("Crawling single directory '%s'"), path);
+		str = g_strdup_printf ("Crawling single directory '%s'", path);
 	}
 
 	g_message ("%s", str);
 
-	g_object_set (fs, "status", str, NULL);
+	/* Always set the progress here to at least 1% */
+	g_object_set (fs,
+	              "progress", 0.01,
+	              "status", str,
+	              NULL);
 	g_free (str);
 	g_free (path);
 
@@ -2515,7 +2759,7 @@ crawl_directories_cb (gpointer user_data)
 	}
 
 	/* Directory couldn't be processed */
-	directory_data_free (fs->private->current_directory);
+	directory_data_unref (fs->private->current_directory);
 	fs->private->current_directory = NULL;
 
 	return TRUE;
@@ -2569,6 +2813,50 @@ crawl_directories_stop (TrackerMinerFS *fs)
 	}
 }
 
+static gboolean
+should_recurse_for_directory (TrackerMinerFS *fs,
+			      GFile          *file)
+{
+	gboolean recurse = FALSE;
+	GList *dirs;
+
+	for (dirs = fs->private->config_directories; dirs; dirs = dirs->next) {
+		DirectoryData *data;
+
+		data = dirs->data;
+
+		if (data->recurse &&
+		    (g_file_equal (file, data->file) ||
+		     g_file_has_prefix (file, data->file))) {
+			/* File is inside a recursive dir */
+			recurse = TRUE;
+			break;
+		}
+	}
+
+	return recurse;
+}
+
+/* This function is for internal use, adds the file to the processing
+ * queue with the same directory settings than the corresponding
+ * config directory.
+ */
+static void
+tracker_miner_fs_directory_add_internal (TrackerMinerFS *fs,
+					 GFile          *file)
+{
+	DirectoryData *data;
+	gboolean recurse;
+
+	recurse = should_recurse_for_directory (fs, file);
+	data = directory_data_new (file, recurse);
+
+	fs->private->directories =
+		g_list_append (fs->private->directories, data);
+
+	crawl_directories_start (fs);
+}
+
 /**
  * tracker_miner_fs_directory_add:
  * @fs: a #TrackerMinerFS
@@ -2582,12 +2870,19 @@ tracker_miner_fs_directory_add (TrackerMinerFS *fs,
                                 GFile          *file,
                                 gboolean        recurse)
 {
+	DirectoryData *dir_data;
+
 	g_return_if_fail (TRACKER_IS_MINER_FS (fs));
 	g_return_if_fail (G_IS_FILE (file));
 
+	dir_data = directory_data_new (file, recurse);
+
+	fs->private->config_directories =
+		g_list_append (fs->private->config_directories, dir_data);
+
 	fs->private->directories =
 		g_list_append (fs->private->directories,
-		               directory_data_new (file, recurse));
+			       directory_data_ref (dir_data));
 
 	crawl_directories_start (fs);
 }
@@ -2659,12 +2954,28 @@ tracker_miner_fs_directory_remove (TrackerMinerFS *fs,
 
 		if (g_file_equal (file, data->file) ||
 		    g_file_has_prefix (file, data->file)) {
-			directory_data_free (data);
+			directory_data_unref (data);
 			fs->private->directories = g_list_delete_link (fs->private->directories, link);
 			return_val = TRUE;
 		}
 	}
 
+	dirs = fs->private->config_directories;
+
+	while (dirs) {
+		DirectoryData *data = dirs->data;
+		GList *link = dirs;
+
+		dirs = dirs->next;
+
+		if (g_file_equal (file, data->file)) {
+			directory_data_unref (data);
+			fs->private->config_directories = g_list_delete_link (fs->private->config_directories, link);
+			return_val = TRUE;
+		}
+	}
+
+
 	/* Remove anything contained in the removed directory
 	 * from all relevant processing queues.
 	 */
@@ -2903,3 +3214,20 @@ tracker_miner_fs_get_parent_urn (TrackerMinerFS *fs,
 
 	return data->parent_urn;
 }
+
+void
+tracker_miner_fs_force_recheck (TrackerMinerFS *fs)
+{
+	GList *directories;
+
+	g_return_if_fail (TRACKER_IS_MINER_FS (fs));
+
+	g_message ("Forcing re-check on all index directories");
+
+	directories = g_list_copy (fs->private->config_directories);
+	g_list_foreach (directories, (GFunc) directory_data_ref, NULL);
+
+	fs->private->directories = g_list_concat (fs->private->directories, directories);
+
+	crawl_directories_start (fs);
+}
