@@ -469,7 +469,7 @@ msoffice_convert_and_normalize_chunk (guint8    *buffer,
 		g_free (converted_text);
 		g_free (normalized_chunk);
 	} else {
-		g_warning ("Couldn't convert %d bytes from %s to UTF-8: %s",
+		g_warning ("Couldn't convert %" G_GSIZE_FORMAT " bytes from %s to UTF-8: %s",
 		           chunk_size,
 		           is_ansi ? "CP1252" : "UTF-16",
 		           error ? error->message : "no error given");
@@ -858,7 +858,7 @@ extract_msword_content (GsfInfile *infile,
 	guint8 *piece_table = NULL;
 	guint8 *clx = NULL;
 	gint lcb_piece_table;
-	gint piece_count;
+	gint piece_count = 0;
 	gint32 fc;
 	GString *content = NULL;
 	guint8 *text_buffer = NULL;
@@ -1305,8 +1305,8 @@ xls_get_extended_record_string (GsfInput  *stream,
 	guint parsing_record = 0;
 	guint8 tmp_buffer[4] = { 0 };
 	guint i;
-	guint8 *buffer;
-	gsize buffer_size;
+	guint8 *buffer = NULL;
+	gsize buffer_size = 0;
 
 	/* Parsing the record from the list */
 	record = &g_array_index (list, ExcelExtendedStringRecord, parsing_record);
